package pl.edu.pwr.gui;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import org.hamcrest.Matchers;
import org.junit.Assert;
import org.junit.Test;

public class Iteracja3Test {


	public static String INTENSITY_2_ASCII_10CHARS = "@%#*+=-:. ";
	public static String INTENSITY_2_ASCII_70CHARS = "$@B%8&WM#*oahkbdpqwmZO0QLCJUYXzcvunxrjft/\\|()1{}[]?-_+~<>i!lI;:,\"^`'. ";
	
	@Test
	public void test1() {
		shouldBeProperlyConvertedInternal("iteracja3_test1.txt", INTENSITY_2_ASCII_10CHARS);
		shouldBeProperlyScaledInternal("iteracja3_test1.txt", 10, 1);
	}
	
	@Test
	public void test2() {
		shouldBeProperlyConvertedInternal("iteracja3_test2.txt", INTENSITY_2_ASCII_70CHARS);
		shouldBeProperlyScaledInternal("iteracja3_test2.txt", 70, 1);
	}
	
	@Test
	public void test3() {
		shouldBeProperlyConvertedInternal("iteracja3_test3.txt", new StringBuilder(INTENSITY_2_ASCII_10CHARS).reverse().toString());
		shouldBeProperlyScaledInternal("iteracja3_test3.txt", 10, 1);
	}
	
	@Test
	public void test4() {
		shouldBeProperlyConvertedInternal("iteracja3_test4.txt", new StringBuilder(INTENSITY_2_ASCII_70CHARS).reverse().toString());
		shouldBeProperlyScaledInternal("iteracja3_test4.txt", 70, 1);
	}
	
	
	private void shouldBeProperlyConvertedInternal(String filename, String expected) {
		InputStream resource = this.getClass().getClassLoader().getResourceAsStream(filename);
		if (resource == null) {
			Assert.fail("Input file not found");
		}
		try (InputStreamReader reader = new InputStreamReader(resource)) {
			try (BufferedReader br = new BufferedReader(reader)) {
				try {
					String line = br.readLine();
					Assert.assertThat(line, Matchers.is(Matchers.equalTo(expected)));
				} catch (IOException e) {
					e.printStackTrace();
					Assert.fail("Something went terribly wrong");
				}
			}
		} catch (IOException e1) {
			e1.printStackTrace();
		}
	}
	
	private char getCharacter(String filename) {
		InputStream resource = this.getClass().getClassLoader().getResourceAsStream(filename);
		if (resource == null) {
			Assert.fail("Input file not found");
		}
		try (BufferedReader br = new BufferedReader(new InputStreamReader(resource))) {
			return (char) br.read();
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		return 0;
	}
	
	@Test
	public void test5() {
		String filename = "palms_80.txt";
		shouldBeProperlyScaledInternal(filename, 80, 60);
		Assert.assertThat(getCharacter(filename), Matchers.is(Matchers.equalTo('-')));
	}
	
	@Test
	public void test6() {
		String filename = "palms_160.txt";
		shouldBeProperlyScaledInternal(filename, 160, 120);
		Assert.assertThat(getCharacter(filename), Matchers.is(Matchers.equalTo('(')));
	}
	
	@Test
	public void test8() {
		String filename = "palms.txt";
		shouldBeProperlyScaledInternal(filename, 1024, 768);
		Assert.assertThat(getCharacter(filename), Matchers.is(Matchers.equalTo('(')));
	}
		
	@Test
	public void test7() {
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		double width = screenSize.getWidth();
		double height = Math.round(width * 0.75);
		
		shouldBeProperlyScaledInternal("palms_full.txt", (int) width, (int) height);
	}
	
	private void shouldBeProperlyScaledInternal(String filename, int expectedWidth, int expectedHeight) {
		InputStream resource = this.getClass().getClassLoader().getResourceAsStream(filename);
		if (resource == null) {
			Assert.fail("Input file not found");
		}
		int lines = 0;
		try (InputStreamReader reader = new InputStreamReader(resource)) {
			try (BufferedReader br = new BufferedReader(reader)) {
				try {
					String line;
					while ((line  = br.readLine()) != null) {
						Assert.assertThat(line.length(), Matchers.is(Matchers.equalTo(expectedWidth)));	
						lines++;
					}
				} catch (IOException e) {
					e.printStackTrace();
					Assert.fail("Something went terribly wrong");
				}
			}
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		Assert.assertThat(lines, Matchers.is(expectedHeight));
	}
	
}
